<?php

namespace EventPortal\Domain\Form;

use EventPortal\Domain\Component\Button;
use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Domain\Event\Form\Models\Form as FormModel;
use EventPortal\Domain\Event\Form\Models\FormStep;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Enums\InertiaMethod;

class EditForm
{
    /** @var \EventPortal\Domain\Delegate\Models\Delegate */
    private $delegate;

    /** @var \EventPortal\Domain\Event\Form\Models\Form */
    private $formModel;

    /** @var \EventPortal\Domain\Event\Models\Event */
    private $event;

    /** @var int */
    private $previousStepOrder;

    /** @var \EventPortal\Domain\Event\Form\Models\FormStep */
    private $currentStep;

    public static function make(): self
    {
        return new self();
    }

    public function forDelegate(Delegate $delegate): self
    {
        $this->delegate = $delegate;

        return $this;
    }

    public function forForm(FormModel $form): self
    {
        $form->loadMissing([
            'steps.sections.questions',
        ]);

        $this->formModel = $form;

        return $this;
    }

    public function forEvent(Event $event): self
    {
        $this->event = $event;

        return $this;
    }

    public function setPreviousStep(?FormStep $previousStep = null): self
    {
        $this->previousStepOrder = optional($previousStep)->order;

        return $this;
    }

    public function currentStep(?FormStep $currentStep = null): self
    {
        $this->currentStep = $currentStep;

        return $this;
    }

    public function render(): array
    {
        $form = $this->formModel;
        $previousStep = $this->previousStepOrder;
        $currentStep = $this->currentStep;

        return \EventPortal\Domain\Form\Facades\Form::makeFromDatabase($form, $currentStep)
            ->model($this->delegate)
            ->context($this->event, $currentStep)
            ->addActions(static function (Event $event) use ($form, $currentStep, $previousStep): array {
                if ($currentStep->isFirst() === false) {
                    $buttons[] = Button::make('Back')
                        ->variant('secondary')
                        ->method(InertiaMethod::VISIT())
                        ->route('delegate.form.index', [
                            'event' => $event->slug,
                            'form' => $form->slug,
                            'stepOrder' => $previousStep,
                        ]);
                }

                $buttons[] = Button::make('Save for later')
                    ->variant('secondary')
                    ->method(InertiaMethod::POST())
                    ->route('delegate.form.update', [
                        'form' => $form->slug,
                        'event' => $event->slug,
                        'step' => $currentStep,
                    ]);

                return $buttons;
            })
            ->addPrimaryAction(static function (Event $event) use ($form, $currentStep): Button {
                return Button::make($currentStep->isLast() ? 'Finish' : 'Next step')
                    ->method(InertiaMethod::POST())
                    ->route('delegate.form.store', [
                        'event' => $event->slug,
                        'form' => $form->slug,
                        'step' => $currentStep,
                    ]);
            })
            ->render();
    }
}
