<?php

namespace EventPortal\Http\Controllers\Admin;

use EventPortal\Concerns\EventRequiresReApproval;
use EventPortal\Domain\Agenda\Actions\CreateSession;
use EventPortal\Domain\Agenda\Actions\DeleteSession;
use EventPortal\Domain\Agenda\Actions\UpdateSession;
use EventPortal\Domain\Agenda\AgendasForBuilder;
use EventPortal\Domain\Agenda\Models\Agenda;
use EventPortal\Domain\Agenda\Models\Pathway;
use EventPortal\Domain\Agenda\Models\Session;
use EventPortal\Http\Controllers\Controller;
use Illuminate\Http\JsonResponse;

class SessionController extends Controller
{
    use EventRequiresReApproval;

    public function __construct()
    {
        $this->middleware('check-permission:agenda.session.store')->only('store');
        $this->middleware('check-permission:agenda.session.update')->only('update');
        $this->middleware('check-permission:agenda.session.destroy')->only('destroy');
    }

    public function store(Agenda $agenda, Pathway $pathway, CreateSession $createSession): JsonResponse
    {
        $createSession->execute($pathway, $agenda, request()->input());

        toast()->success(trans('session.created'));

        $event = $agenda->event()->first();

        if ($event !== null && $this->doesEventRequireReApproval($event)) {
            $event->updateApprovalStatusToRequireResubmission();
        }

        return response()->json([
            'success' => true,
            'agendas' => AgendasForBuilder::get($agenda->event),
        ]);
    }

    public function update(Agenda $agenda, Session $session, UpdateSession $updateSession): JsonResponse
    {
        $updateSession->execute($session, $agenda, request()->input());

        toast()->success(trans('session.updated'));

        $event = $agenda->event()->first();

        if ($event !== null && $this->doesEventRequireReApproval($event)) {
            $event->updateApprovalStatusToRequireResubmission();
        }

        return response()->json([
            'success' => true,
            'agendas' => AgendasForBuilder::get($agenda->event),
        ]);
    }

    public function delete(Agenda $agenda, Session $session, DeleteSession $deleteSession): JsonResponse
    {
        $deleteSession->execute($session);

        toast()->success(trans('role.deleted'));

        return response()->json([
            'success' => true,
            'agendas' => AgendasForBuilder::get($agenda->event),
        ]);
    }
}
