<?php

namespace EventPortal\Tests\Feature\Admin;

use EventPortal\Domain\Admin\Models\Admin;
use EventPortal\Tests\Support\Traits\SeedRolesAndPermissions;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;

class CannotPerformActionOnSelfTest extends TestCase
{
    use RefreshDatabase, SeedRolesAndPermissions;

    /** @test **/
    public function cannot_perform_action_on_self(): void
    {
        $admin = factory(Admin::class)->create();

        $this
            ->actingAsSuperAdmin($admin)
            ->get(route('admin.user.index'));

        $this
            ->post(route('admin.user.delete', ['admin' => $admin]))
            ->assertRedirect(route('admin.user.index'));

        $this
            ->assertDatabaseHas('users', [
                'id' => $admin->user->id,
            ])
            ->assertDatabaseHas('admins', [
                'id' => $admin->id,
            ]);
    }

    /** @test **/
    public function cannot_perform_action_on_another(): void
    {
        $admin = factory(Admin::class)->create();
        $user = $admin->user;

        $this
            ->actingAsSuperAdmin()
            ->get(route('admin.user.index'));

        $this
            ->post(route('admin.user.delete', ['admin' => $admin]))
            ->assertRedirect(route('admin.user.index'));

        $this
            ->assertDatabaseMissing('users', [
                'id' => $user->id,
            ])
            ->assertDatabaseMissing('admins', [
                'id' => $admin->id,
            ]);
    }
}
