<?php

namespace EventPortal\Tests\Feature\Admin\Event;

use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Team\Models\Team;
use EventPortal\Enums\EmailType;
use EventPortal\Tests\Support\Traits\SeedRolesAndPermissions;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;

class CreateEventTest extends TestCase
{
    use RefreshDatabase, SeedRolesAndPermissions;

    /** @test **/
    public function default_emails_are_created_with_an_event()
    {
        $this
            ->actingAsSuperAdmin()
            ->followingRedirects()
            ->post(route('admin.event.store'), [
                'name' => 'Test Event',
                'team_id' => factory(Team::class)->create()->id,
                'email' => 'event@email.com',
            ])
            ->assertOk();

        $event = Event::first();
        $numberOfEmailTypes = count(EmailType::getAll());
        $this->assertCount($numberOfEmailTypes, $event->emails);
    }

    /** @test **/
    public function slug_auto_generated_from_name()
    {
        $this->withoutExceptionHandling();

        $this
            ->actingAsSuperAdmin()
            ->followingRedirects()
            ->post(route('admin.event.store'), [
                'name' => 'This is my test event',
                'team_id' => factory(Team::class)->create()->id,
                'email' => 'event@email.com',
            ])
            ->assertOk();

        $event = Event::first();
        $this->assertEquals('this-is-my-test-event', $event->slug);
    }
}
