<?php

namespace EventPortal\Tests\Unit\EventDelegate\Handlers;

use EventPortal\Domain\Delegate\Models\DelegateFormStep;
use EventPortal\Domain\Event\Form\Enums\State;
use EventPortal\Domain\Event\Form\Models\Form;
use EventPortal\Domain\Event\Form\Models\FormStep;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Event\Models\EventDelegate;
use EventPortal\Domain\EventDelegate\Handlers\UpdatePartialProgress;
use EventPortal\Domain\Form\Payload;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Pipeline\Pipeline;

class UpdatePartialProgressTest extends TestCase
{
    use RefreshDatabase;

    /** @test **/
    public function updates_partial_form_step_to_partial(): void
    {
        $event = factory(Event::class)->create();
        $form = factory(Form::class)->create(['event_id' => $event->id]);
        $step = factory(FormStep::class)->create(['form_id' => $form->id]);
        $eventDelegate = factory(EventDelegate::class)->create(['event_id' => $event->id]);

        $payload = Payload::make()
            ->withContext(['eventDelegate' => $eventDelegate, 'formStep' => $step]);

        app(Pipeline::class)
            ->send($payload)
            ->through([new UpdatePartialProgress()])
            ->thenReturn();

        $this
            ->assertDatabaseHas('delegate_form_step', [
                'delegate_id' => $eventDelegate->delegate->id,
                'form_step_id' => $step->id,
                'state' => State::Partial(),
            ]);
    }

    /** @test **/
    public function does_not_mark_previous_steps_as_partial(): void
    {
        $event = factory(Event::class)->create();
        $form = factory(Form::class)->create(['event_id' => $event->id]);
        $stepA = factory(FormStep::class)->create(['form_id' => $form->id]);
        $stepB = factory(FormStep::class)->create(['form_id' => $form->id]);

        $eventDelegate = factory(EventDelegate::class)->create(['event_id' => $event->id]);

        factory(DelegateFormStep::class)->create([
            'form_step_id' => $stepA->id,
            'delegate_id' => $eventDelegate->delegate->id,
            'state' => State::Complete(),
        ]);

        factory(DelegateFormStep::class)->create([
            'form_step_id' => $stepB->id,
            'delegate_id' => $eventDelegate->delegate->id,
            'state' => State::Partial(),
        ]);

        $payload = Payload::make()
            ->withContext([
                'eventDelegate' => $eventDelegate,
                'formStep' => $stepA,
            ]);

        app(Pipeline::class)
            ->send($payload)
            ->through([new UpdatePartialProgress()])
            ->thenReturn();

        $this
            ->assertDatabaseHas('delegate_form_step', [
                'delegate_id' => $eventDelegate->delegate->id,
                'form_step_id' => $stepA->id,
                'state' => State::Complete(),
            ])
            ->assertDatabaseHas('delegate_form_step', [
                'delegate_id' => $eventDelegate->delegate->id,
                'form_step_id' => $stepB->id,
                'state' => State::Partial(),
            ]);
    }
}
